-----------------------
-- NPSee             --
-- By: Wire @ Byriel --
-----------------------

TargetManager = class();

TargetManager.Defaults = {
  Spacing = 2,
  Interrupt = 0.06,
  CastBarSpacing = true,
  CastBarDisplay = true,
  PetDisplay = false,
  PvPDisplay = true,
  Stacking = false
};

function TargetManager:Constructor(application)
  
  -- parameters
  self:SetApplication(application);
  self:SetParent(self:GetApplication():GetContext());
  self:SetIdle(self:GetApplication():GetIdle());
  
  -- cascade bar
  self:SetCascadeBar(Wired.UI.CascadeBar(
    "Wire's " .. self:GetApplication():GetName() .. " Targets", self:GetApplication():GetContext()
  ));
  
  -- defaults
  self:ApplyTargetManagerDefaults();
  
  -- idle
  self:GetIdle().Wired.Event.Listener:Add(Wired.Event.Listener(
    "Idle",
    function()
      self:Idle();
    end,
    true
  ));
  
  -- units changed
  table.insert(Event.Unit.Available, {
    function(value)
      self:UnitsChanged(value);
    end,
    self:GetApplication():GetName(),
    "Units Changed Handler"
  });
  
  -- persistence
  
  -- cast bar spacing
  self.castBarSpacing = Wired.Persistence.PersistentVariable(
    "CastBarSpacing",
     self:GetParent():GetBase(),
     TargetManager.Defaults.CastBarSpacing
  );
  
  -- cast bar display
  self.castBarDisplay = Wired.Persistence.PersistentVariable(
    "CastBarDisplay",
     self:GetParent():GetBase(),
     TargetManager.Defaults.CastBarDisplay
  );
  
  -- pet display
  self.petDisplay = Wired.Persistence.PersistentVariable(
    "PetDisplay",
     self:GetParent():GetBase(),
     TargetManager.Defaults.PetDisplay
  );
  
  -- PvP display
  self.PvPDisplay = Wired.Persistence.PersistentVariable(
    "PvPDisplay",
     self:GetParent():GetBase(),
     TargetManager.Defaults.PvPDisplay
  );
  
  -- stacking
  self.stacking = Wired.Persistence.PersistentVariable(
    "Stacking",
     self:GetParent():GetBase(),
     TargetManager.Defaults.Stacking
  );
  
  -- target persistence
  self.targetPersistence = Wired.Persistence.PersistentVariable(
    "Targets", self:GetParent():GetBase()
  );
end

-- application
function TargetManager:GetApplication()
  return self.application;
end

function TargetManager:SetApplication(value)
  self.application = value;
end

-- parent
function TargetManager:GetParent()
  return self.parent;
end

function TargetManager:SetParent(value)
  self.parent = value;
end

-- idle
function TargetManager:GetIdle()
  return self.idle;
end

function TargetManager:SetIdle(value)
  self.idle = value;
end

-- cascade bar
function TargetManager:GetCascadeBar()
  return self.cascadeBar;
end

function TargetManager:SetCascadeBar(value)
  self.cascadeBar = value;
end

-- cast bar spacing
function TargetManager:GetCastBarSpacing()
  return self.castBarSpacing:GetValue();
end

function TargetManager:SetCastBarSpacing(value)
  self.castBarSpacing:SetValue(value);
  
  -- redraw
  self:RedrawGauges();
end

-- cast bar display
function TargetManager:GetCastBarDisplay()
  return self.castBarDisplay:GetValue();
end

function TargetManager:SetCastBarDisplay(value)
  self.castBarDisplay:SetValue(value);
  
  -- redraw
  self:RedrawGauges();
end

-- pet display
function TargetManager:GetPetDisplay()
  return self.petDisplay:GetValue();
end

function TargetManager:SetPetDisplay(value)
  self.petDisplay:SetValue(value);
  
  -- redraw
  self:RedrawGauges();
end

-- PvP display
function TargetManager:GetPvPDisplay()
  return self.PvPDisplay:GetValue();
end

function TargetManager:SetPvPDisplay(value)
  self.PvPDisplay:SetValue(value);
  
  -- redraw
  self:RedrawGauges();
end

-- stacking
function TargetManager:GetStacking()
  return self.stacking:GetValue();
end

function TargetManager:SetStacking(value)
  self.stacking:SetValue(value);
  
  -- redraw gauges
  self:RedrawGauges();
end

-- target persistence
function TargetManager:GetTargetPersistence()
  if (type(self.targetPersistence:GetValue()) ~= "table") then
    self.targetPersistence:SetValue({});
  end
  return self.targetPersistence:GetValue();
end

function TargetManager:SetTargetPersistence(value)
  self.targetPersistence:SetValue(value);
end

-- gauges
function TargetManager:GetGauges()
  if (type(self.gauges) ~= "table") then
    self.gauges = {};
  end
  return self.gauges;
end

function TargetManager:SetGauges(value)
  self.gauges = value;
end

-- defaults
function TargetManager:ApplyTargetManagerDefaults()
  
end

-- resource
function TargetManager:GetResource(details)
  local result = {};
  
  -- caster
  if (
    (details.calling == "cleric") or (details.calling == "mage")
  ) then
    result = {
      name = "Mana",
      color = Wired.UI.Color.Gauge.Mana,
      max = details.manaMax,
      value = details.mana
    };
    
  -- rogue
  elseif (details.calling == "rogue") then
    result = {
      name = "Energy",
      color = Wired.UI.Color.Gauge.Power,
      max = details.energyMax,
      value = details.energy
    };
    
  -- warrior
  elseif (details.calling == "warrior") then
    result = {
      name = "Power",
      color = Wired.UI.Color.Gauge.Energy,
      max = 100,
      value = details.power
    };
  end
  
  return result;
end

function TargetManager:GetGaugeData(details, resource, cast)
  local result = {};
  
  result.health = {
    max = details.healthMax,
    value = details.health
  };
  result.resource = {
    max = resource.max,
    value = resource.value
  };
  --[[
  result.cast = {
    max = cast.duration,
    value = cast.remaining
  };
  --]]
  return result;
end

function TargetManager:UpdateGauge(gauge, details, resource)
  
  -- data
  local data = self:GetGaugeData(details, resource);
  for id, gauge in pairs(gauge:GetGauges()) do
    
    -- rescale
    if (gauge:GetScale() ~= data[id].max) then
      gauge:Reset(gauge:GetTitle(), data[id].max, data[id].value);
      
    -- adjust value
    elseif (gauge:GetValue() ~= data[id].value) then
      gauge:SetValue(data[id].value);
    end
  end
  
  -- update mark
  if (details.mark ~= gauge:GetMark()) then
    gauge:SetMark(details.mark);
  end
end

-- add unit
function TargetManager:AddUnit(unit, details)

  -- check for recycled gauge
  local gauges = 1;
  local gauge = nil;
  for id, recycledGauge in pairs(self:GetGauges()) do
    gauges = gauges + 1;
    if ((not gauge) and (not recycledGauge:GetVisible())) then
      gauge = recycledGauge;
      self:GetGauges()[id] = nil;
    end
  end
  
  -- create new
  if (not gauge) then
    gauge = Wired.UI.TargetGauge(
      "Gauge_" .. gauges,
      self:GetParent()
    );
    
    -- persistence
    gauge.Wired.Event.Listener:Add(Wired.Event.Listener(
      "LockedChanged",
      function(sender)
        self:LockedChanged(sender);
      end,
      true
    ));
  end
  
  -- reindex
  self:GetGauges()[unit] = gauge;
  
  -- resource
  local resource = self:GetResource(details);
  
  -- reset
  gauge:Reset(details.name, details.healthMax, resource.name, resource.color, resource.max);
  
  -- update gauge
  self:UpdateGauge(gauge, details, resource);
  
  -- load gauge
  local persistence = self:GetTargetPersistence()[details.name];
  if (persistence) then
    
    -- size
    gauge:SetSize(persistence.size);
    
    -- offset
    gauge:SetPosition(persistence.left, persistence.top);
    
    -- lock
    gauge:SetLocked(true);
    
  -- unlock
  else
    gauge:SetLocked(false);
  end
  
  -- show
  gauge:SetVisible(true);
end

-- units changed
function TargetManager:UnitsChanged(value)
  
  -- enabled
  if (self:GetApplication():GetEnabled()) then
    
    -- redraw
    local redraw = false;
    
    -- units
    for unit, v in pairs(value) do
      
      -- details
      local details = Inspect.Unit.Detail(unit);
      
      -- gauge exists
      if (self:GetGauges()[unit]) then
      
        -- disable unacquired
        if (not details) then
          self:GetGauges()[unit]:SetVisible(false);
          
          -- redraw
          redraw = true;
        
        -- enable live reacquired
        elseif ((not self:GetGauges()[unit]:GetVisible()) and (details.health > 0)) then
          self:GetGauges()[unit]:SetVisible(true);
          
          -- redraw
          redraw = true;
        end
        
      -- create for valid targets
      elseif (self:ValidTarget(details)) then
        
        -- add unit
        self:AddUnit(unit, details);
        
        -- redraw
        redraw = true;
      end
    end
    
    -- redraw
    if (redraw) then
      self:RedrawGauges();
    end
  end
end

-- valid target
function TargetManager:ValidTarget(details)
  local result = false;
  
  -- details
  if (details) then
    
    -- alive hostile
    if ((details.relation == "hostile") and (details.health > 0)) then
      
      -- pet
      local showPet = true;
      if (details.nameSecondary) then
        if (string.match(details.nameSecondary, "'s Pet")) then
          showPet = self:GetPetDisplay();
        end
      end
      
      -- npc or not player if not pvp display or not pet if not pet display
      if ((self:GetPvPDisplay() or (not details.player)) and showPet) then
        result = true;
      end
    end
  end
  return result;
end

-- idle
function TargetManager:Idle()

  -- redraw
  local redraw = false;
  
  -- gauges
  for unit, gauge in pairs(self:GetGauges()) do
    
    -- active
    if (gauge:GetVisible()) then
      
      -- details
      local details = Inspect.Unit.Detail(unit);
      
      -- disable
      if (not details) then
        gauge:SetVisible(false);
        
        -- redraw
        redraw = true;
        
      -- disable invalid target mobs
      elseif (not self:ValidTarget(details)) then
        gauge:SetVisible(false);
        
        -- redraw
        redraw = true;
          
      -- update valid target mobs
      else
        self:UpdateGauge(gauge, details, self:GetResource(details));
        
        -- displaying cast bars
        if ((not gauge:GetLocked()) or self:GetCastBarDisplay()) then
          
          -- cast
          local castGauge = gauge:GetCastGauge();
          local cast = Inspect.Unit.Castbar(unit);
          
          -- check interrupted
          if (not cast) then
            
            -- was visible
            if (castGauge:GetVisible()) then
              
              -- continue fade
              if (castGauge:GetFade()) then
                castGauge:Fade(self:GetIdle())
                
              -- start interrupted
              elseif (
                (castGauge:GetValue() > TargetManager.Defaults.Interrupt) and
                (castGauge:GetValue() < castGauge:GetScale())
              ) then
                castGauge:SetFlipped(false);
                castGauge:Reset("INTERRUPTED", 1);
                castGauge:Fade();
                
              -- unavailable
              else
                castGauge:SetVisible(false);
              end
              
              -- redraw
              if (
                (not castGauge:GetVisible()) and
                (not self:GetCastBarSpacing())
              ) then
                redraw = true;
              end
            end
          
          -- update cast
          else
            
            -- stop fade
            if (castGauge:GetFade()) then
              castGauge:StopFade();
            end
            
            -- reset
            if (castGauge:GetScale() ~= cast.duration) then
            
              -- flip
              castGauge:SetFlipped(not cast.channeled);
              
              -- color
              if (cast.uninterruptible) then
                castGauge:SetColor(Wired.UI.Color.Gauge.Uninterruptible);
              else
                castGauge:SetColor(Wired.UI.Color.Gauge.Cast);
              end
              
              -- reset
              castGauge:Reset(string.upper(cast.abilityName), cast.duration);
              
            -- update
            elseif (cast.remaining ~= castGauge:GetValue()) then
              castGauge:SetValue(cast.remaining);
            end
            
            -- show
            if (not castGauge:GetVisible()) then
              castGauge:SetVisible(true);
              
              -- redraw
              if (not self:GetCastBarSpacing()) then
                redraw = true;
              end
            end
          end
        end
      end
    end
  end
  
  -- redraw
  if (redraw) then
    self:RedrawGauges();
  end
end

function TargetManager:RedrawGauges()
  
  -- top
  local top = self:GetCascadeBar():GetBar();
  
  -- gauges
  for unit, gauge in pairs(self:GetGauges()) do
    
    -- only draw visible and unlocked
    if (gauge:GetVisible() and (not gauge:GetLocked())) then
      
      -- relative size
      local ratio = Wired.UI.TargetGauge.Defaults.Size / Wired.UI.CascadeBar.Defaults.Size;
      local relativeSize = math.floor(0.5 + self:GetCascadeBar():GetSize() * ratio);
      
      -- resize
      if (gauge:GetSize() ~= relativeSize) then
        gauge:SetSize(relativeSize);
      end
      
      -- cast resize
      local castSpacing = 0;
      if (
        self:GetCastBarDisplay() and
        (self:GetCastBarSpacing() or gauge:GetCastGauge():GetVisible())
      ) then
        castSpacing = gauge:GetCastGauge():GetSize();
      end
      
      -- reposition
      if (self:GetStacking()) then
        -- override
        if (top == self:GetCascadeBar():GetBar()) then
          castSpacing = 0;
        end
        gauge:SetPosition(
          self:GetCascadeBar():GetBar():GetLeft(),
          top:GetTop() - gauge:GetBase():GetHeight() - TargetManager.Defaults.Spacing - castSpacing
        );
      else
        gauge:SetPosition(
          self:GetCascadeBar():GetBar():GetLeft(),
          top:GetBottom() + TargetManager.Defaults.Spacing + castSpacing
        );
      end
      
      -- top
      top = gauge:GetBase();
    end
  end
end

-- locked changed
function TargetManager:LockedChanged(value)
  if (value:GetLocked()) then
    self:GetTargetPersistence()[value:GetHealthGauge():GetTitle()] = {
      size = value:GetSize(),
      left = value:GetBase():GetLeft(),
      top = value:GetBase():GetTop(),
    };
  else
    self:GetTargetPersistence()[value:GetHealthGauge():GetTitle()] = nil;
  end
  
  -- redraw
  self:RedrawGauges();
end
