-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.UI.CascadeBar = class();

-- constants
Wired.UI.CascadeBar.Constants = {
  Texture = Wired.UI.Texture("Wired", "UI/Resources/gauge.png"),
  Size = {
    Min = 10,
    Max = 150,
    Ratio = 12
  }
};

-- defaults
Wired.UI.CascadeBar.Defaults = {
  Size = 25,
  Width = 300,
  Height = 25,
  Colors = {
    Bar = {
      Unlocked = Wired.UI.Color.Yellow,
      Locked = Wired.UI.Color.Red
    },
    Font = Wired.UI.Color.Black
  },
  Visible = {
    Bar = true,
    TitleLabel = true,
    Mask = false
  },
  Alpha = {
    MouseOver = 0.6,
    MouseOut = 0
  }
};

function Wired.UI.CascadeBar:Constructor(name, parent)
  Wire(self);
  
  -- parameters
  self:SetName(name);
  self:SetParent(parent);
  
  -- bar
  self:SetBar(Wired.UI.Trion.Texture(self:GetName() .. "_CascadeBar", self:GetParent()));
  self:GetBar():SetVisible(false);
  self:GetBar():SetTexture(Wired.UI.CascadeBar.Constants.Texture);
  
  -- title label
  self:SetTitleLabel(Wired.UI.Trion.Text(self:GetName() .. "_TitleLabel", self:GetBar()));
  self:GetTitleLabel():SetLayer(1);
  self:GetTitleLabel():SetPoint({child = "CENTER", parent = "CENTER"}, {x = 0, y = 0});
  
  -- mask
  self:SetMask(Wired.UI.Trion.Texture(self:GetName() .. "_Mask", self:GetBar()));
  self:GetMask():SetTexture(Wired.UI.CascadeBar.Constants.Texture);
  self:GetMask():SetLayer(1);
  self:GetMask():SetPoint({child = "CENTER", parent = "CENTER"}, {x = 0, y = 0});
  
  -- drag
  self:GetBar():GetBase().Wired.UI.Drag:Enable(true);
  
  -- persistence
  self:GetBar():GetBase().Wired.Persistence:Enable(self:GetName() .. "_CascadeBar");
  
  -- locked
  self.locked = Wired.Persistence.PersistentVariable(
    "Locked", self:GetBar():GetBase(), false
  );
  
  -- x
  self.x = Wired.Persistence.PersistentVariable(
    "X", self:GetBar():GetBase(), 100
  );
  
  -- y
  self.y = Wired.Persistence.PersistentVariable(
    "Y", self:GetBar():GetBase(), 100
  );
  
  -- size
  self.size = Wired.Persistence.PersistentVariable(
    "Size", self:GetBar():GetBase(), Wired.UI.CascadeBar.Defaults.Size
  );
  
  -- defaults
  self:ApplyCascadeBarDefaults();
    
  -- load
  self:Load();
  
  -- events
  
  -- move
  self:GetBar():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "Move",
    function()
      self:Move();
    end
  ));
  
  -- right click bar
  self:GetBar():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "RightClick",
    function(sender, value)
      self:RightClick();
    end
  ));
  
  -- right click mask
  self:GetMask():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "RightClick",
    function(sender, value)
      self:RightClick();
    end
  ));
  
  -- mouse in bar
  self:GetMask():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "MouseIn",
    function(sender, value)
      self:SetMouseOver(true);
    end
  ));
  
  -- mosue out bar
  self:GetMask():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "MouseOut",
    function(sender, value)
      self:SetMouseOver(false);
    end
  ));
  
  -- zoom back
  self:GetBar():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "WheelBack",
    function()
      self:Wheel(1);
    end
  ));
  
  -- zoom forward
  self:GetBar():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "WheelForward",
    function()
      self:Wheel(-1);
    end
  ));
end

-- name
function Wired.UI.CascadeBar:GetName()
  return self.name;
end

function Wired.UI.CascadeBar:SetName(value)
  self.name = value;
end

-- parent
function Wired.UI.CascadeBar:GetParent()
  return self.parent;
end

function Wired.UI.CascadeBar:SetParent(value)
  self.parent = value;
end

-- bar
function Wired.UI.CascadeBar:GetBar()
  return self.bar;
end

function Wired.UI.CascadeBar:SetBar(value)
  self.bar = value;
end

-- title Label
function Wired.UI.CascadeBar:GetTitleLabel()
  return self.titleLabel;
end

function Wired.UI.CascadeBar:SetTitleLabel(value)
  self.titleLabel = value;
end

-- mask
function Wired.UI.CascadeBar:GetMask()
  return self.mask;
end

function Wired.UI.CascadeBar:SetMask(value)
  self.mask = value;
end

-- title
function Wired.UI.CascadeBar:GetTitle()
  return self:GetTitleLabel():GetText();
end

function Wired.UI.CascadeBar:SetTitle(value)
  self:GetTitleLabel():SetText(value);
  
  -- layout
  self:Layout();
end

-- size
function Wired.UI.CascadeBar:GetSize()
  return self.size:GetValue();
end

function Wired.UI.CascadeBar:SetSize(value)
  
  -- restrict
  self.size:SetValue(math.max(
    Wired.UI.CascadeBar.Constants.Size.Min,
    math.min(Wired.UI.CascadeBar.Constants.Size.Max, value)
  ));
  
  -- layout
  self:Layout();
end

-- locked
function Wired.UI.CascadeBar:GetLocked()
  return self.locked:GetValue();
end

function Wired.UI.CascadeBar:SetLocked(value)
  self.locked:SetValue(value);
  
  -- drag
  self:GetBar():GetBase().Wired.UI.Drag.data.enabled = not self.locked:GetValue();
  
  -- update
  self:Update();
end

-- mouseOver
function Wired.UI.CascadeBar:GetMouseOver()
  return self.mouseOver;
end

function Wired.UI.CascadeBar:SetMouseOver(value)
  self.mouseOver = value;

  -- update
  self:Update();
end

-- visible
function Wired.UI.CascadeBar:GetVisible()
  return self:GetBar():GetVisible();
end

function Wired.UI.CascadeBar:SetVisible(value)
  self:GetBar():SetVisible(value);
end

-- width
function Wired.UI.CascadeBar:GetWidth()
  return self:GetBar():GetWidth();
end

-- height
function Wired.UI.CascadeBar:GetHeight()
  return self:GetBar():GetHeight();
end

-- defaults
function Wired.UI.CascadeBar:ApplyCascadeBarDefaults()
  
  -- font colors
  self:GetTitleLabel():SetFontColor(Wired.UI.CascadeBar.Defaults.Colors.Font);
  
  -- values
  self:SetTitle(self:GetName());
  
  -- layout
  self:Layout();
  
  -- visibile
  self:GetBar():SetVisible(Wired.UI.CascadeBar.Defaults.Visible.Bar);
  self:GetTitleLabel():SetVisible(Wired.UI.CascadeBar.Defaults.Visible.TitleLabel);
  self:GetMask():SetVisible(Wired.UI.CascadeBar.Defaults.Visible.Mask);
  
  -- update
  self:Update();
end

-- layout
function Wired.UI.CascadeBar:Layout()
  
  -- bar
  self:GetBar():SetWidth(self:GetSize() * Wired.UI.CascadeBar.Constants.Size.Ratio);
  self:GetBar():SetHeight(self:GetSize());
  
  -- mask
  self:GetMask():SetWidth(self:GetSize() * Wired.UI.CascadeBar.Constants.Size.Ratio);
  self:GetMask():SetHeight(self:GetSize());
  
  -- title label
  self:GetTitleLabel():SetHeight(self:GetBar():GetHeight());
  self:GetTitleLabel():ScaleFontSize();
  self:GetTitleLabel():SetWidth(self:GetTitleLabel():GetFullWidth());
end

-- set position
function Wired.UI.CascadeBar:SetPosition(left, top)
  self:GetBar():SetPosition(left, top);
end

-- events
function Wired.UI.CascadeBar:Move()
  self.x:SetValue(self:GetBar():GetBase():GetLeft());
  self.y:SetValue(self:GetBar():GetBase():GetTop());
end

-- load
function Wired.UI.CascadeBar:Load()
  
  -- position
  if (
    (self.x:GetValue() ~= self:GetBar():GetBase():GetLeft()) or
    (self.y:GetValue() ~= self:GetBar():GetBase():GetTop())
  ) then
    self:GetBar():SetPosition(self.x:GetValue(), self.y:GetValue());
  end
  
  -- lock
  self:SetLocked(self:GetLocked());
end

-- right click
function Wired.UI.CascadeBar:RightClick()
  self:SetLocked(not self:GetLocked());
end

-- update
function Wired.UI.CascadeBar:Update()
  
  -- mouseover
  if (self:GetMouseOver() or (not self:GetLocked())) then
    self:GetBar():SetAlpha(Wired.UI.CascadeBar.Defaults.Alpha.MouseOver);
    self:GetTitleLabel():SetVisible(true);
  elseif (self:GetLocked()) then
    self:GetBar():SetAlpha(Wired.UI.CascadeBar.Defaults.Alpha.MouseOut);
    self:GetTitleLabel():SetVisible(false);
  end
  
  -- locked
  if (self:GetLocked()) then
    self:GetMask():SetVisible(true);
    self:GetBar():SetColor(Wired.UI.CascadeBar.Defaults.Colors.Bar.Locked);
  else
    self:GetMask():SetVisible(false);
    self:GetBar():SetColor(Wired.UI.CascadeBar.Defaults.Colors.Bar.Unlocked);
  end
end

-- wheel
function Wired.UI.CascadeBar:Wheel(value)
  if (not self:GetLocked()) then
    self:SetSize(self:GetSize() + value);
  end
end
