-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.UI.Check = class();

-- defaults
Wired.UI.Check.Defaults = {
  Color = {
    Base = {
      Checked = Wired.UI.Color.Grey,
      Unchecked = Wired.UI.Color.Grey
    },
    Box = {
      Checked = Wired.UI.Color.Green,
      Unchecked = Wired.UI.Color.Red
    }
  },
  Checked = false,
  Size = 20,
  Padding = 1,
  Visible = {
    Base = false,
    Box = true
  }
};

function Wired.UI.Check:Constructor(name, parent)
  
  -- wire
  Wire(self);
  
  -- parameters
  self:SetName(name);
  self:SetParent(parent);
  
  -- base
  self:SetBase(Wired.UI.Trion.Frame(self:GetName() .. "_Base", self:GetParent()));
  self:GetBase():SetVisible(false);
  
  -- box
  self:SetBox(Wired.UI.Trion.Frame(self:GetName() .. "_Box", self:GetBase()));
  self:GetBox():SetVisible(false);
  self:GetBox():SetPoint({child = "CENTER", parent = "CENTER"}, {x = 0, y = 0});
  
  -- left click base
  self:GetBase():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "LeftClick",
    function(sender, value)
      self:LeftClick();
    end
  ));
  
  -- defaults
  self:ApplyCheckDefaults();
end

-- name
function Wired.UI.Check:GetName()
  return self.name;
end

function Wired.UI.Check:SetName(value)
  self.name = value;
end

-- parent
function Wired.UI.Check:GetParent()
  return self.parent;
end

function Wired.UI.Check:SetParent(value)
  self.parent = value;
end

-- base
function Wired.UI.Check:GetBase()
  return self.base;
end

function Wired.UI.Check:SetBase(value)
  self.base = value;
end

-- box
function Wired.UI.Check:GetBox()
  return self.box;
end

function Wired.UI.Check:SetBox(value)
  self.box = value;
end

-- padding
function Wired.UI.Check:GetPadding()
  if (not self.padding) then
    self.padding = 0;
  end
  return self.padding;
end

function Wired.UI.Check:SetPadding(value)
  self.padding = value;
  
  -- update size
  self:UpdateSize();
end

-- size
function Wired.UI.Check:GetSize()
  return self.size;
end

function Wired.UI.Check:SetSize(value)
  self.size = value;
  
  -- update size
  self:UpdateSize();
end

-- value
function Wired.UI.Check:GetChecked()
  return self.checked;
end

function Wired.UI.Check:SetChecked(value)
  self.checked = value;
  
  -- update
  self:Update();
  
  -- fire checked changed event
  if (type(self.Wired.Event.CheckedChanged) == "function") then
    self.Wired.Event:CheckedChanged();
  end
end

-- visible
function Wired.UI.Check:GetVisible()
  return self:GetBase():GetVisible();
end

function Wired.UI.Check:SetVisible(value)
  self:GetBase():SetVisible(value);
  
  -- fire visible changed event
  if (type(self.Wired.Event.VisibleChanged) == "function") then
    self.Wired.Event:VisibleChanged();
  end
end

-- defaults
function Wired.UI.Check:ApplyCheckDefaults()
  
  -- values
  self:SetChecked(Wired.UI.Check.Defaults.Checked);
  
  -- size
  self:SetSize(Wired.UI.Check.Defaults.Size);
  self:SetPadding(Wired.UI.Check.Defaults.Padding);
  
  -- visibile
  self:GetBase():SetVisible(Wired.UI.Check.Defaults.Visible.Base);
  self:GetBox():SetVisible(Wired.UI.Check.Defaults.Visible.Box);
end

-- point
function Wired.UI.Check:SetPoint(anchor, offset)
  self:GetBase():SetPoint(anchor, offset);
end

-- position
function Wired.UI.Check:SetPosition(x, y)
  self:GetBase():SetPosition(x, y);
end

-- left click
function Wired.UI.Check:LeftClick()
  self:SetChecked(not self:GetChecked());
end

-- update size
function Wired.UI.Check:UpdateSize()
  
  -- bar
  self:GetBase():SetWidth(self:GetSize());
  self:GetBase():SetHeight(self:GetSize());
  
  -- box
  self:GetBox():SetWidth(self:GetSize() - (self:GetPadding() * 2));
  self:GetBox():SetHeight(self:GetSize() - (self:GetPadding() * 2));
end

-- update
function Wired.UI.Check:Update()
  
  -- checked
  if (self:GetChecked()) then
    self:GetBase():SetColor(Wired.UI.Check.Defaults.Color.Base.Checked);
    self:GetBox():SetColor(Wired.UI.Check.Defaults.Color.Box.Checked);
  else
    self:GetBase():SetColor(Wired.UI.Check.Defaults.Color.Base.Unchecked);
    self:GetBox():SetColor(Wired.UI.Check.Defaults.Color.Box.Unchecked);
  end
end
