-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.UI.Pin = class();

-- defaults
Wired.UI.Pin.Defaults = {
  Width = 40,
  Height = 40,
  Visible = {
    Base = true
  }
};

function Wired.UI.Pin:Constructor(name, parent)
  
  -- wire
  Wire(self);
  
  -- parameters
  self:SetName(name);
  self:SetParent(parent);
  
  -- bar
  self:SetBase(Wired.UI.Trion.Texture(self:GetName() .. "_Pin", self:GetParent()));
  self:GetBase():SetVisible(false);
  
  -- drag
  self:GetBase():GetBase().Wired.UI.Drag:Enable(true);
  
  -- persistence
  self:GetBase():GetBase().Wired.Persistence:Enable(self:GetName());
  
  -- locked
  self.locked = Wired.Persistence.PersistentVariable(
    "Locked", self:GetBase():GetBase(), false
  );
  
  -- x
  self.x = Wired.Persistence.PersistentVariable(
    "X", self:GetBase():GetBase(), 100
  );
  
  -- y
  self.y = Wired.Persistence.PersistentVariable(
    "Y", self:GetBase():GetBase(), 100
  );
  
  -- defaults
  self:ApplyPinDefaults();
    
  -- load
  self:Load();
  
  -- events
  
  -- move
  self:GetBase():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "Move",
    function()
      self:Move();
    end
  ));
  
  -- left click base
  self:GetBase():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "StationaryClick",
    function(sender, value)
      self:LeftClick();
    end,
    true
  ));
  
  -- right click base
  self:GetBase():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "RightClick",
    function(sender, value)
      self:RightClick();
    end
  ));
  
  -- mouse in base
  self:GetBase():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "MouseIn",
    function(sender, value)
      self:SetMouseOver(true);
    end
  ));
  
  -- mosue out base
  self:GetBase():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "MouseOut",
    function(sender, value)
      self:SetMouseOver(false);
    end
  ));
end

-- name
function Wired.UI.Pin:GetName()
  return self.name;
end

function Wired.UI.Pin:SetName(value)
  self.name = value;
end

-- parent
function Wired.UI.Pin:GetParent()
  return self.parent;
end

function Wired.UI.Pin:SetParent(value)
  self.parent = value;
end

-- base
function Wired.UI.Pin:GetBase()
  return self.base;
end

function Wired.UI.Pin:SetBase(value)
  self.base = value;
end

-- locked
function Wired.UI.Pin:GetLocked()
  return self.locked:GetValue();
end

function Wired.UI.Pin:SetLocked(value)
  self.locked:SetValue(value);
  
  -- drag
  self:GetBase():GetBase().Wired.UI.Drag.data.enabled = not self.locked:GetValue();
  
  -- update
  self:Update();
end

-- mouse over
function Wired.UI.Pin:GetMouseOver()
  return self.mouseOver;
end

function Wired.UI.Pin:SetMouseOver(value)
  self.mouseOver = value;

  -- update
  self:Update();
end

-- engaged
function Wired.UI.Pin:GetEngaged()
  return self.engaged;
end

function Wired.UI.Pin:SetEngaged(value)
  self.engaged = value;

  -- update
  self:Update();
end

-- textures
function Wired.UI.Pin:GetTextures()
  return self.textures;
end

function Wired.UI.Pin:SetTextures(value)
  self.textures = value;
  
  -- update
  self:Update();
  
  -- resize
  self:GetBase():ResizeToTexture();
end

-- defaults
function Wired.UI.Pin:ApplyPinDefaults()
    
  -- size
  self:GetBase():SetWidth(Wired.UI.Pin.Defaults.Width);
  self:GetBase():SetHeight(Wired.UI.Pin.Defaults.Height);
  
  -- visibile
  self:GetBase():SetVisible(Wired.UI.Pin.Defaults.Visible.Base);
  
  -- update
  self:Update();
end

-- events
function Wired.UI.Pin:Move()
  self.x:SetValue(self:GetBase():GetBase():GetLeft());
  self.y:SetValue(self:GetBase():GetBase():GetTop());
end

-- load
function Wired.UI.Pin:Load()
  
  -- position
  if (
    (self.x:GetValue() ~= self:GetBase():GetBase():GetLeft()) or
    (self.y:GetValue() ~= self:GetBase():GetBase():GetTop())
  ) then
    self:GetBase():SetPosition(self.x:GetValue(), self.y:GetValue());
  end
  
  -- lock
  self:SetLocked(self:GetLocked());
end

-- left click
function Wired.UI.Pin:LeftClick()
  if (type(self.Wired.Event.Clicked) == "function") then
    self.Wired.Event:Clicked();
  end
end

-- right click
function Wired.UI.Pin:RightClick()
  self:SetLocked(not self:GetLocked());
end

-- update
function Wired.UI.Pin:Update()
  
  -- textures
  if (self:GetTextures()) then
  
    local texture = nil;
    
    -- on
    if (self:GetMouseOver()) then
      if (self:GetEngaged()) then
        texture = self:GetTextures().engagedOn
      elseif (self:GetLocked()) then
        texture = self:GetTextures().lockedOn
      else
        texture = self:GetTextures().unlockedOn
      end
      
    -- off
    else
      if (self:GetEngaged()) then
        texture = self:GetTextures().engagedOff
      elseif (self:GetLocked()) then
        texture = self:GetTextures().lockedOff
      else
        texture = self:GetTextures().unlockedOff
      end
    end
    
    -- update texture
    self:GetBase():SetTexture(texture);
  end
end
