-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.UI.Tab = class();

Wired.UI.Tab.Constants = {
  
};

Wired.UI.Tab.Defaults = {
  Spacing = 2
};

function Wired.UI.Tab:Constructor(name, parent)
  
  -- wire
  Wire(self);
  
  -- parameters
  self:SetName(name);
  self:SetParent(parent);
  
  -- tab list
  self:SetTabList(Wired.UI.List("TabList", self:GetParent()));
  self:GetTabList():SetOrientation(Wired.UI.List.Constants.Orientation.Horizontal);
  self:GetTabList():SetVisible(true);
  
  -- defaults
  self:ApplyTabDefaults();
end

-- name
function Wired.UI.Tab:GetName()
  return self.name;
end

function Wired.UI.Tab:SetName(value)
  self.name = value;
end

-- parent
function Wired.UI.Tab:GetParent()
  return self.parent;
end

function Wired.UI.Tab:SetParent(value)
  self.parent = value;
end

-- tab list
function Wired.UI.Tab:GetTabList()
  return self.tabList;
end

function Wired.UI.Tab:SetTabList(value)
  self.tabList = value;
end

-- spacing
function Wired.UI.Tab:GetSpacing()
  return self.spacing;
end

function Wired.UI.Tab:SetSpacing(value)
  self.spacing = value;
end

-- tabs
function Wired.UI.Tab:GetTabs()
  if (type(self.tabs) ~= "table") then
    self.tabs = {};
  end
  return self.tabs;
end

function Wired.UI.Tab:SetTabs(value)
  self.tabs = value;
end

function Wired.UI.Tab:AddTab(name, tab)
  
  -- tab
  tab:SetPoint({child = "TOPCENTER", parent = "BOTTOMCENTER"}, {x = 0, y = self:GetSpacing()});
  self:GetTabs()[name] = tab;
  
  -- create cell
  local cell = Wired.UI.Cell(name .. "_TabCell", self:GetTabList());
  cell:SetTitle(name);
  cell:SetVisible(true);
  
  -- left click
  cell.Wired.Event.Listener:Add(Wired.Event.Listener(
    "LeftClick",
    function()
      self:GetTabList():Select(cell);
      self:Select(name);
    end,
    true
  ));
  
  -- add cell
  self:GetTabList():AddCell(cell);
  
  -- scale cells
  self:GetTabList():ScaleCells();
end

-- width
function Wired.UI.Tab:GetWidth()
  self:GetTabList():GetWidth();
end

function Wired.UI.Tab:SetWidth(value)
  self:GetTabList():SetWidth(value);
  
  -- scale cells
  self:GetTabList():ScaleCells();
end

-- defaults
function Wired.UI.Tab:ApplyTabDefaults()
  self:SetSpacing(Wired.UI.Tab.Defaults.Spacing);
end

-- set point
function Wired.UI.Tab:SetPoint(anchor, offset)
  self:GetTabList():SetPoint(anchor, offset);
end

-- select
function Wired.UI.Tab:Select(tabName)
  for name, tab in pairs(self:GetTabs()) do
    if (name == tabName) then
      tab:SetVisible(true);
    else
      tab:SetVisible(false);
    end
  end
end
