-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.UI.TargetGauge = class();

-- constants
Wired.UI.TargetGauge.Constants = {
  Size = {
    Health = 25 / 28,
    Resource = 3 / 28,
    Cast = 15 / 28,
    Min = 25,
    Max = 150
  }
};

-- defaults
Wired.UI.TargetGauge.Defaults = {
  Colors = {
    Base = Wired.UI.Color.Clear,
    Font = {
      CastGauge = Wired.UI.Color.White
    }
  },
  Visible = {
    Base = false,
    HealthGauge = {
      Gauge = true,
      Value = true,
      Title = true,
      Mark = true
    },
    ResourceGauge = {
      Gauge = true,
      Value = false,
      Title = false,
      Mark = false
    },
    CastGauge = {
      Gauge = true,
      Value = false,
      Title = true,
      Mark = false
    }
  },
  Size = 28
};

function Wired.UI.TargetGauge:Constructor(name, parent)
  
  -- wire
  Wire(self);
  
  -- parameters
  self:SetName(name);
  self:SetParent(parent);
    
  -- base
  self:SetBase(Wired.UI.Trion.Frame(self:GetName() .. "_Base", self:GetParent()));
  self:GetBase():SetVisible(false);
  self:GetBase():SetPoint({child = "TOPLEFT", parent = "TOPLEFT"}, {x = 0, y = 0});
  
  -- health gauge
  self:SetHealthGauge(Wired.UI.Gauge(self:GetName() .. "_HealthGauge", self:GetBase()));
  self:GetHealthGauge():SetVisible(false);
  self:GetHealthGauge():SetPoint({child = "TOPCENTER", parent = "TOPCENTER"}, {x = 0, y = 0});
  
  -- resource gauge
  self:SetResourceGauge(Wired.UI.Gauge(self:GetName() .. "_ResourceGauge", self:GetBase()));
  self:GetResourceGauge():SetVisible(false);
  self:GetResourceGauge():SetPoint({child = "BOTTOMCENTER", parent = "BOTTOMCENTER"}, {x = 0, y = 0});
  
  -- cast gauge
  self:SetCastGauge(Wired.UI.Gauge(self:GetName() .. "_CastGauge", self:GetBase()));
  self:GetCastGauge():SetVisible(false);
  self:GetCastGauge():SetPoint({child = "BOTTOMCENTER", parent = "TOPCENTER"}, {x = 0, y = 0});
  
  -- apply defaults
  self:ApplyTargetGaugeDefaults();
  
  -- events
  
  -- drag
  self:GetBase():GetBase().Wired.UI.Drag:Enable(true);
  
  -- zoom forward
  self:GetBase():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "RightClick",
    function()
      self:RightClick();
    end
  ));
  
  -- zoom back
  self:GetBase():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "WheelBack",
    function()
      self:Wheel(1);
    end
  ));
  
  -- zoom forward
  self:GetBase():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "WheelForward",
    function()
      self:Wheel(-1);
    end
  ));
end

-- name
function Wired.UI.TargetGauge:GetName()
  return self.name;
end

function Wired.UI.TargetGauge:SetName(value)
  self.name = value;
end

-- parent
function Wired.UI.TargetGauge:GetParent()
  return self.parent;
end

function Wired.UI.TargetGauge:SetParent(value)
  self.parent = value;
end

-- base
function Wired.UI.TargetGauge:GetBase()
  return self.base;
end

function Wired.UI.TargetGauge:SetBase(value)
  self.base = value;
end

-- health gauge
function Wired.UI.TargetGauge:GetHealthGauge()
  return self.healthGauge;
end

function Wired.UI.TargetGauge:SetHealthGauge(value)
  self.healthGauge = value;
end

-- resource gauge
function Wired.UI.TargetGauge:GetResourceGauge()
  return self.resourceGauge;
end

function Wired.UI.TargetGauge:SetResourceGauge(value)
  self.resourceGauge = value;
end

-- cast gauge
function Wired.UI.TargetGauge:GetCastGauge()
  return self.castGauge;
end

function Wired.UI.TargetGauge:SetCastGauge(value)
  self.castGauge = value;
end

-- gauges
function Wired.UI.TargetGauge:GetGauges()
  return {
    health = self:GetHealthGauge(),
    resource = self:GetResourceGauge(),
    --cast = self:GetCastGauge()
  };
end

-- size
function Wired.UI.TargetGauge:GetSize()
  return self.size;
end

function Wired.UI.TargetGauge:SetSize(value)
  self.size = value;
  
  -- update size
  self:UpdateSize();
end

-- resource color
function Wired.UI.TargetGauge:GetResourceColor()
  return self:GetResourceGauge():GetColor();
end

function Wired.UI.TargetGauge:SetResourceColor(value)
  self:GetResourceGauge():SetColor(value);
end

-- visible
function Wired.UI.TargetGauge:GetVisible()
  return self:GetBase():GetVisible();
end

function Wired.UI.TargetGauge:SetVisible(value)
  self:GetBase():SetVisible(value);
  
  -- fire visible changed event
  if (type(self.Wired.Event.VisibleChanged) == "function") then
    self.Wired.Event:VisibleChanged();
  end
end

-- mark
function Wired.UI.TargetGauge:GetMark()
  return self:GetHealthGauge():GetMark();
end

function Wired.UI.TargetGauge:SetMark(value)
  self:GetHealthGauge():SetMark(value);
end

-- locked
function Wired.UI.TargetGauge:GetLocked()
  return self.locked
end

function Wired.UI.TargetGauge:SetLocked(value)
  self.locked = value;
  
  -- drag
  self:GetBase():GetBase().Wired.UI.Drag.data.enabled = not self.locked;
  
  -- fire locked event
  if (type(self.Wired.Event.LockedChanged) == "function") then
    self.Wired.Event:LockedChanged();
  end
end

-- defaults
function Wired.UI.TargetGauge:ApplyTargetGaugeDefaults()
  
  -- base color
  self:GetBase():SetColor(Wired.UI.TargetGauge.Defaults.Colors.Base);
  self:GetCastGauge():SetFontColor(Wired.UI.TargetGauge.Defaults.Colors.Font.CastGauge);
  
  -- reset
  self:SetSize(Wired.UI.TargetGauge.Defaults.Size);
  
  -- base
  self:GetBase():SetVisible(Wired.UI.TargetGauge.Defaults.Visible.Base);
  
  -- health gauge visible
  self:GetHealthGauge():SetVisible(Wired.UI.TargetGauge.Defaults.Visible.HealthGauge.Gauge);
  self:GetHealthGauge():GetTitleLabel():SetVisible(Wired.UI.TargetGauge.Defaults.Visible.HealthGauge.Title);
  self:GetHealthGauge():GetValueLabel():SetVisible(Wired.UI.TargetGauge.Defaults.Visible.HealthGauge.Value);
  self:GetHealthGauge():GetMarkLabel():SetVisible(Wired.UI.TargetGauge.Defaults.Visible.HealthGauge.Mark);
  
  -- resource gauge visible
  self:GetResourceGauge():SetVisible(Wired.UI.TargetGauge.Defaults.Visible.ResourceGauge.Gauge);
  self:GetResourceGauge():GetTitleLabel():SetVisible(Wired.UI.TargetGauge.Defaults.Visible.ResourceGauge.Title);
  self:GetResourceGauge():GetValueLabel():SetVisible(Wired.UI.TargetGauge.Defaults.Visible.ResourceGauge.Value);
  self:GetResourceGauge():GetMarkLabel():SetVisible(Wired.UI.TargetGauge.Defaults.Visible.ResourceGauge.Mark);
  
  -- cast gauge visible
  self:GetCastGauge():SetVisible(Wired.UI.TargetGauge.Defaults.Visible.CastGauge.Gauge);
  self:GetCastGauge():GetTitleLabel():SetVisible(Wired.UI.TargetGauge.Defaults.Visible.CastGauge.Title);
  self:GetCastGauge():GetValueLabel():SetVisible(Wired.UI.TargetGauge.Defaults.Visible.CastGauge.Value);
  self:GetCastGauge():GetMarkLabel():SetVisible(Wired.UI.TargetGauge.Defaults.Visible.CastGauge.Mark);
end

-- position
function Wired.UI.TargetGauge:SetPosition(x, y)
  self:GetBase():SetPosition(x, y);
end

-- reset
function Wired.UI.TargetGauge:Reset(healthTitle, healthScale, resourceTitle, resourceColor, resourceScale)
  
  -- health
  self:GetHealthGauge():Reset(healthTitle, healthScale);
  
  -- resource
  if (resourceTitle and resourceColor and resourceScale) then
    self:GetResourceGauge():Reset(resourceTitle, resourceScale);
    self:GetResourceGauge():SetColor(resourceColor);
    if (not self:GetResourceGauge():GetVisible()) then
      self:GetResourceGauge():SetVisible(true);
    end
  else
    self:GetResourceGauge():SetVisible(false);
  end
  
  -- cast
  self:GetCastGauge():SetVisible(false);
  
  -- show
  self:SetVisible(true);
end

-- update size
function Wired.UI.TargetGauge:UpdateSize()
  
  -- sizes
  local size = self:GetSize();
  local resource = math.max(1, math.floor(
    0.5 + size * Wired.UI.TargetGauge.Constants.Size.Resource
  ));
  local health = size - resource;
  local cast = math.max(1, math.floor(
    0.5 + size * Wired.UI.TargetGauge.Constants.Size.Cast
  ));
  
  -- size
  self:GetBase():SetHeight(size);
  self:GetHealthGauge():SetSize(health);
  self:GetResourceGauge():SetSize(resource);
  self:GetCastGauge():SetSize(cast);
  
  -- width
  self:GetResourceGauge():SetStaticWidth(self:GetHealthGauge():GetWidth());
  self:GetCastGauge():SetStaticWidth(self:GetHealthGauge():GetWidth());
  self:GetBase():SetWidth(self:GetHealthGauge():GetWidth());
end

-- right click
function Wired.UI.TargetGauge:RightClick()
  self:SetLocked(not self:GetLocked());
end

-- wheel
function Wired.UI.TargetGauge:Wheel(value)
  if (not self:GetLocked()) then
    self:SetSize(self:GetSize() + value);
  end
end
