-----------------------
-- Wired             --
-- By: Wire @ Byriel --
-----------------------

Wired.UI.Timer = class(Wired.UI.Gauge);

-- defaults
Wired.UI.Timer.Defaults = {
  Wheel = 1,
  Size = 25,
  Transient = true
};

function Wired.UI.Timer:Constructor(name, parent, application)  
  Wired.UI.Gauge.Constructor(self, name, parent);
  
  -- parameters
  self:SetApplication(application);
  self:SetIdle(application:GetIdle());
  
  -- apply defaults
  self:ApplyTimerDefaults();
  
  -- events
  
  -- drag
  self:GetBase():GetBase().Wired.UI.Drag:Enable(true);
  
  -- idle
  self:GetIdle().Wired.Event.Listener:Add(Wired.Event.Listener(
    "Idle",
    function(sender, value)
      self:Idle();
    end,
    true
  ));
  
  -- zoom forward
  self:GetBase():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "RightClick",
    function()
      self:RightClick();
    end
  ));
  
  -- zoom back
  self:GetBase():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "WheelBack",
    function()
      self:Wheel(Wired.UI.Timer.Defaults.Wheel);
    end
  ));
  
  -- zoom forward
  self:GetBase():GetBase().Wired.Event.Listener:Add(Wired.Event.Listener(
    "WheelForward",
    function()
      self:Wheel(-Wired.UI.Timer.Defaults.Wheel);
    end
  ));
end

-- application
function Wired.UI.Timer:GetApplication()
  return self.application;
end

function Wired.UI.Timer:SetApplication(value)
  self.application = value;
end

-- idle
function Wired.UI.Timer:GetIdle()
  return self.idle;
end

function Wired.UI.Timer:SetIdle(value)
  self.idle = value;
end

-- active
function Wired.UI.Timer:GetActive()
  return self.active;
end

function Wired.UI.Timer:SetActive(value)
  self.active = value;
end

-- continuous
function Wired.UI.Timer:Continuous()
  return self.continuous;
end

function Wired.UI.Timer:SetContinuous(value)
  self.continuous = value;
end

-- transient
function Wired.UI.Timer:GetTransient()
  return self.transient;
end

function Wired.UI.Timer:SetTransient(value)
  self.transient = value;
end

-- locked
function Wired.UI.Timer:GetLocked()
  return self.locked
end

function Wired.UI.Timer:SetLocked(value)
  self.locked = value;
  
  -- drag
  self:GetBase():GetBase().Wired.UI.Drag.data.enabled = not self.locked;
  
  -- fire locked event
  if (type(self.Wired.Event.LockedChanged) == "function") then
    self.Wired.Event:LockedChanged();
  end
end

-- defaults
function Wired.UI.Timer:ApplyTimerDefaults()
  
  -- transient
  self:SetTransient(Wired.UI.Timer.Defaults.Transient);
end

-- position
function Wired.UI.Timer:SetPosition(x, y)
  self:GetBase():SetPosition(x, y);
end

-- idle
function Wired.UI.Timer:Idle()
  
  -- active
  if (self:GetActive()) then
    
    -- update value
    self:SetValue(self:GetValue() - self:GetIdle():GetElapsed());
    if (self:GetValue() <= 0) then
      
      -- restart
      if (self:Continuous()) then
        self:ResetTimer();
        
      -- leave on screen
      elseif (not self:GetTransient()) then
        self:SetActive(false);
        
      -- stop
      else
        self:SetActive(false);
        self:SetVisible(false);
      end
      
      -- fire finished event
      if (type(self.Wired.Event.TimerFinished) == "function") then
        self.Wired.Event:TimerFinished();
      end
    end
  end
end

-- reset timer
function Wired.UI.Timer:ResetTimer()
  self:SetValue(self:GetScale());
  self:SetVisible(true);
end

-- right click
function Wired.UI.Timer:RightClick()
  self:SetLocked(not self:GetLocked());
end

-- wheel
function Wired.UI.Timer:Wheel(value)
  if (not self:GetLocked()) then
    self:SetSize(self:GetSize() + value);
  end
end
